@@ -7,6 +7,7 @@ text and metadata from PDFs as well.
 You can read the full docs at https://pypdf.readthedocs.io/.
 """
 
+from ._crypt_providers import crypt_provider
 from ._encryption import PasswordType
 from ._merger import PdfFileMerger, PdfMerger
 from ._page import PageObject, Transformation
@@ -16,8 +17,20 @@ from ._writer import ObjectDeletionFlag, PdfFileWriter, PdfWriter
 from .pagerange import PageRange, parse_filename_page_ranges
 from .papersizes import PaperSize
 
+try:
+    import PIL
+
+    pil_version = PIL.__version__
+except ImportError:
+    pil_version = "none"
+
+_debug_versions = (
+    f"pypdf=={__version__}, crypt_provider={crypt_provider}, PIL={pil_version}"
+)
+
 __all__ = [
     "__version__",
+    "_debug_versions",
     "PageRange",
     "PaperSize",
     "DocumentInformation",
